"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.toSimpleTransitionAnimations = exports.toToggleAnimations = exports.toTransitionAnimation = exports.toPersistedAnimation = void 0;
const AnimationInput_1 = require("./AnimationInput");
/**
 * Returns an edited animation that runs it's last keyframe infinitely (until interrupted)
 * Useful for controlled animations that are tied to state rather than transitions
 *
 * The edits are:
 * - set iterations: 'Infinity' on the last Keyframe in the Keyframes list
 * - set fill: 'forwards' on the Animation options
 */
const toPersistedAnimation = (toPersist) => {
    const { keyframes, options } = (0, AnimationInput_1.normalizedAnimation)(toPersist);
    if (!keyframes.length) {
        return { keyframes: [], options: options };
    }
    const lastKeyframe = keyframes.slice(-1)[0];
    const rest = keyframes.slice(0, -1);
    const persistedLast = {
        ...lastKeyframe,
        iterations: 'Infinity',
    };
    const persistentOptions = {
        ...options,
        fill: 'forwards',
    };
    return {
        keyframes: [...rest, persistedLast],
        options: persistentOptions,
    };
};
exports.toPersistedAnimation = toPersistedAnimation;
/**
 * Returns an edited animation that retains the style of its last keyframe
 * Useful for controlled animations that are tied to transitions so that when the transition is complete, the style persists in the new state
 *
 * The edits are:
 * - set fill: 'forwards' on the Animation options
 */
const toTransitionAnimation = (animation) => {
    const { keyframes, options } = (0, AnimationInput_1.normalizedAnimation)(animation);
    return {
        keyframes: keyframes,
        options: { fill: 'forwards', ...options },
    };
};
exports.toTransitionAnimation = toTransitionAnimation;
/**
 * Helper to create a complete toggling animation by reversing a given togglingOn animation
 *
 * The edits are:
 * - set fill: 'forwards' on the Animation options
 */
const toToggleAnimations = (togglingOn) => {
    const { keyframes, options } = (0, exports.toTransitionAnimation)(togglingOn);
    return {
        togglingOn: { keyframes, options },
        togglingOff: {
            keyframes: keyframes.slice().reverse(),
            options,
        },
    };
};
exports.toToggleAnimations = toToggleAnimations;
/**
 * Helper to create expected SimpleTransition animation Record
 *
 * See {@link hooks.useSimpleTransitioningState}
 *
 * @param transition
 */
const toSimpleTransitionAnimations = (transition) => {
    const edited = (0, exports.toTransitionAnimation)(transition);
    return {
        transition: edited,
    };
};
exports.toSimpleTransitionAnimations = toSimpleTransitionAnimations;
